package com.ylx.massage.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ylx.massage.domain.Coupon;
import com.ylx.massage.domain.vo.CouponReceiveVo;
import com.ylx.massage.domain.vo.CouponReceivesVO;
import com.ylx.massage.mapper.CouponReceiveMapper;
import com.ylx.massage.domain.CouponReceive;
import com.ylx.massage.service.CouponReceiveService;
import com.ylx.massage.service.CouponService;
import com.ylx.massage.utils.DateTimeUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.Lists;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 优惠券领取表(CouponReceive)表服务实现类
 *
 * @author makejava
 * @since 2024-05-14 11:25:45
 */
@Slf4j
@Service("couponReceiveService")
public class CouponReceiveServiceImpl extends ServiceImpl<CouponReceiveMapper, CouponReceive> implements CouponReceiveService {

    @Resource
    private CouponService couponService;

    @Resource
    private CouponReceiveMapper couponReceiveMapper;

    @Override
    public Coupon submit(CouponReceive couponReceive) {
        LambdaQueryWrapper<CouponReceive> queryWrapper = new LambdaQueryWrapper<>();
        Coupon coupon = couponService.getById(couponReceive.getCouponId());
        if (coupon == null) {
            return null;
        }
        queryWrapper.eq(CouponReceive::getOpenid, couponReceive.getOpenid()).eq(CouponReceive::getCouponId, couponReceive.getCouponId());
        //限制领取数量
        List<CouponReceive> one = this.list(queryWrapper);
        if (coupon.getUserLimit() <= one.size()) {
            return null;
        }
        couponReceive.setExpirationTime(DateTimeUtils.addDays(new Date(), coupon.getTermDays()));
        this.save(couponReceive);
        return coupon;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer submits(CouponReceivesVO couponReceive) {

        LambdaQueryWrapper<Coupon> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.in(Coupon::getId, couponReceive.getCouponIds());

        List<Coupon> list = couponService.list(queryWrapper);
        if (CollectionUtil.isEmpty(list)) {
            log.info("优惠卷不存在");
            return null;
        }
        ArrayList<CouponReceive> couponReceives = Lists.newArrayList();
        list.forEach(coupon -> {
            CouponReceive couponRec= new CouponReceive();
            couponRec.setId(Long.toString(IdWorker.getId(couponRec)));
            couponRec.setCouponId(coupon.getId());
            couponRec.setOpenid(couponReceive.getOpenId());
            couponRec.setDeptId(coupon.getDeptId());
            couponRec.setDeptName(coupon.getDeptName());
            couponRec.setUseState(0);
            couponRec.setExpirationTime(DateTimeUtils.addDays(new Date(), coupon.getTermDays()));
            couponReceives.add(couponRec);
        });
        return couponReceiveMapper.insertBatch(couponReceives);
    }

    @Override
    public List<CouponReceiveVo> getByOpenId(String openid) {
        List<CouponReceiveVo> CouponReceiveVos = couponReceiveMapper.getByOpenId(openid);
        CouponReceiveVos.forEach(couponReceiveVo -> {
            couponReceiveVo.setExpirationTimeString(DateTimeUtils.formatDate(couponReceiveVo.getExpirationTime(), DateTimeUtils.DATE_FORMAT));
        });
        return CouponReceiveVos;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void removeCoupons(List<CouponReceiveVo> coupons) {

        boolean b = this.removeByIds(coupons.stream().map(CouponReceiveVo::getId).collect(Collectors.toList()));
        //使用数量+1
        coupons.forEach(coup -> {
            UpdateWrapper<Coupon> wrapper = new UpdateWrapper<>();
            wrapper.lambda().eq(Coupon::getId, coup.getCouponId());
            wrapper.setSql(" used_num = used_num + " + 1);
            couponService.update(wrapper);
        });
    }

    @Override
    public List<Coupon> couponWindows(CouponReceive couponReceive) {
        //判断是否领过优惠卷
        //查询领取优惠卷
        List<CouponReceiveVo> CouponReceiveVos = couponReceiveMapper.getByOpenId(couponReceive.getOpenid());
        //如果没有领取优惠卷
        if (CollectionUtil.isEmpty(CouponReceiveVos)) {
            //查询优惠卷
            LambdaQueryWrapper<Coupon> couponLambdaQueryWrapper = new LambdaQueryWrapper<>();
            couponLambdaQueryWrapper.eq(Coupon::getStatus, 1).like(Coupon::getDeptName, couponReceive.getDeptName());
            //返回新人优惠卷列表
            return couponService.list(couponLambdaQueryWrapper);
        } else {
            return Collections.emptyList();
        }
    }
}

